#!/usr/bin/env python
# 

from gnuradio import gr
import pmt
import sys
import time

class MsgToFile(gr.sync_block):
  def __init__(self, filename, includeTimestamp,overwrite):
    gr.sync_block.__init__(self, name = "Message To File", in_sig = None, out_sig = None)
    
    self.filename = filename
    self.includeTimestamp = includeTimestamp
    
    try:
      # 0 says unbuffered / write immediately
      if (overwrite):
        self.fp = open(filename,"w",0)
      else:
        self.fp = open(filename,"a",0)
    except Exception as e:
      print("[MsgToFile] Error opening file: %s" % str(e))
      sys.exit(1)
     
    self.message_port_register_in(pmt.intern("msg"))
    self.set_msg_handler(pmt.intern("msg"), self.msgHandler)   

  def msgHandler(self, msg):
    if (self.fp):
      if self.includeTimestamp:
        self.fp.write("["+time.strftime('%H:%M:%S %m/%d/%Y')+"]]\n")
        
      self.fp.write(str(msg)+"\n")
    
  def stop(self):
    if (self.fp):
      try:
        self.fp.close()
      except:
        pass
      
    return True
    


#!/usr/bin/env python

from gnuradio import gr
import pmt

class StateMessageAnd(gr.sync_block):
  def __init__(self):
    gr.sync_block.__init__(self, name = "State Message AND", in_sig = None, out_sig = None)
    
    self.message_port_register_in(pmt.intern("state1"))
    self.message_port_register_in(pmt.intern("state2"))
    self.message_port_register_out(pmt.intern("state"))
    self.set_msg_handler(pmt.intern("state1"), self.state1Handler)   
    self.set_msg_handler(pmt.intern("state2"), self.state2Handler)
    
    self.state1 = False
    self.state2 = False
    
    self.curState = False   

  def state1Handler(self, pdu):
    try:    
      newState = pmt.to_python(pmt.cdr(pdu))
      # print "Received message 1: %d " % newState
      if newState == 1:
        self.state1 = True
      else:
        self.state1 = False
      
      if (self.state1 and self.state2):
        self.sendState(True)
        self.curState = True
      else:
        if (self.curState):
          self.sendState(False)
          self.curState = False
    except Exception as e:
      print("Error with state1 message: %s" % str(e))
      print(str(meta))    
            
  def state2Handler(self, pdu):
    try:    
      newState = pmt.to_python(pmt.cdr(pdu))
      
      # print "Received message 2: %d " % newState
      if newState == 1:
        self.state2 = True
      else:
        self.state2 = False
      
      if (self.state1 and self.state2):
        self.sendState(True)
        self.curState = True
      else:
        if (self.curState):
          self.sendState(False)
          self.curState = False
    except Exception as e:
      print("Error with state2 message: %s" % str(e))
      print(str(meta))    
      
  def sendState(self,state):
    if (state):    
      newState = 1
    else:
      newState = 0
      
    self.message_port_pub(pmt.intern("state"),pmt.cons( pmt.intern("state"), pmt.from_long(newState) ))



#!/usr/bin/env python
# 

from gnuradio import gr
import pmt

class StateToBool(gr.sync_block):
  def __init__(self):
    gr.sync_block.__init__(self, name = "State Message To Boolean", in_sig = None, out_sig = None)
    
    self.message_port_register_in(pmt.intern("state"))
    self.message_port_register_out(pmt.intern("bool"))
    self.set_msg_handler(pmt.intern("state"), self.stateHandler)   

  def stateHandler(self, pdu):
    try:    
      newState = pmt.to_python(pmt.cdr(pdu))
      # print "Received message 1: %d " % newState
      if newState == 1:
        boolState = True
      else:
        boolState = False
      
      self.message_port_pub(pmt.intern("bool"),pmt.to_pmt(boolState))
      
    except Exception as e:
      print("[State to Bool] Error with state message: %s" % str(e))
      print(str(meta))    
